#ifndef WRAPPER_HPP
#define WRAPPER_HPP

#include <sys/types.h>
#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>

struct sftp_conn;
struct Attrib;

namespace SFTP {
  /// same as sftp-common.h::Attrib
  struct Attrib {
    //u_int32_t flags; // no flags.
    u_int64_t size;
    u_int32_t uid;
    u_int32_t gid;
    u_int32_t perm;
    u_int32_t atime;
    u_int32_t mtime;
  };

  /// same as sftp-client.h::SFTP_DIRENT
  struct Dirent {
    std::string filename;
    std::string longname;
    Attrib a;
  };
  typedef boost::shared_ptr<Dirent> DirentPtr;

  class Handler {
  public:

    Handler(void);
    bool initialize(const std::string &sshhostpath);
    void terminate(void);

    int download(const std::string &remote, const std::string &local, bool pflag);
    int upload(const std::string &local, const std::string &remote, bool pflag);

    int stat(Attrib *attr, const std::string &remote);
    int lstat(Attrib *attr, const std::string &remote);
    int readdir(std::vector<DirentPtr> *dirent, const std::string &remote);
    int readlink(std::string *remoteReferred, const std::string &remoteLink);
    int symlink(const std::string &linktarget, const std::string &remote);
    int rm(const std::string &remote);
    int mkdir(const std::string &remote);
    int rmdir(const std::string &remote);
    int rename(const std::string &remoteOld, const std::string &remoteNew);
    int setattr(const std::string &remote, const Attrib &attr);

  private:
    void parseSSHHostPath(const std::string &sshhostpath);
    int makeSocket(void);

    void adjustRemotePath(std::string *adjusted, const std::string &remote);

    typedef ::Attrib *(*statfunc_t)(struct sftp_conn *, char *, int);
    template <statfunc_t _F> int internal_stat(Attrib *attr, const std::string &remote);

    sftp_conn *conn;
    std::string hostname;
    std::string basepath;
  };
}

#endif
