#include <sys/types.h>
#include "misc.h"

u_int64_t
get_u64(const void *vp)
{
  const u_char *p = (const u_char *)vp;
  u_int64_t v;

  v  = (u_int64_t)p[0] << 56;
  v |= (u_int64_t)p[1] << 48;
  v |= (u_int64_t)p[2] << 40;
  v |= (u_int64_t)p[3] << 32;
  v |= (u_int64_t)p[4] << 24;
  v |= (u_int64_t)p[5] << 16;
  v |= (u_int64_t)p[6] << 8;
  v |= (u_int64_t)p[7];

  return (v);
}

u_int32_t
get_u32(const void *vp)
{
  const u_char *p = (const u_char *)vp;
  u_int32_t v;

  v  = (u_int32_t)p[0] << 24;
  v |= (u_int32_t)p[1] << 16;
  v |= (u_int32_t)p[2] << 8;
  v |= (u_int32_t)p[3];

  return (v);
}

u_int16_t
get_u16(const void *vp)
{
  const u_char *p = (const u_char *)vp;
  u_int16_t v;

  v  = (u_int16_t)p[0] << 8;
  v |= (u_int16_t)p[1];

  return (v);
}

void
put_u64(void *vp, u_int64_t v)
{
  u_char *p = (u_char *)vp;

  p[0] = (u_char)(v >> 56) & 0xff;
  p[1] = (u_char)(v >> 48) & 0xff;
  p[2] = (u_char)(v >> 40) & 0xff;
  p[3] = (u_char)(v >> 32) & 0xff;
  p[4] = (u_char)(v >> 24) & 0xff;
  p[5] = (u_char)(v >> 16) & 0xff;
  p[6] = (u_char)(v >> 8) & 0xff;
  p[7] = (u_char)v & 0xff;
}

void
put_u32(void *vp, u_int32_t v)
{
  u_char *p = (u_char *)vp;

  p[0] = (u_char)(v >> 24) & 0xff;
  p[1] = (u_char)(v >> 16) & 0xff;
  p[2] = (u_char)(v >> 8) & 0xff;
  p[3] = (u_char)v & 0xff;
}

void
put_u16(void *vp, u_int16_t v)
{
  u_char *p = (u_char *)vp;

  p[0] = (u_char)(v >> 8) & 0xff;
  p[1] = (u_char)v & 0xff;
}
