#include "libsftp.hpp"

#include <sys/stat.h>
#include <iostream>
#include <vector>
#include <string>

namespace {
  void
  waitUntilInput(void)
  {
    std::cout << "\n" << "Please Return" << std::endl;
    std::string tmp;
    std::getline(std::cin, tmp);
  }

  void
  outputError(int error, const std::string name)
  {
    std::cout << "[ERROR] " << name << " (error = " << error << ")\n";
  }
}

int
main(int argc, char *argv[])
{
  std::string sshhostpath = "localhost:/tmp";
  SFTP::Handler sftp;

  if (! sftp.initialize(sshhostpath)) {
    std::cerr << "[ERROR] Couldn't initialize connection to server" << std::endl;
    exit(1);
  }

  waitUntilInput();

  {
    int error = sftp.upload("Makefile", "Makefile", true);
    if (error) outputError(error, "upload1");

    error = sftp.upload("Makefile", "Makefile2", true);
    if (error) outputError(error, "upload2");

    SFTP::Attrib attr;
    error = sftp.stat(&attr, "Makefile2");
    if (error) outputError(error, "stat");
    attr.perm = 0777;
    attr.size = 2048;
    error = sftp.setattr("Makefile2", attr);
  }

  waitUntilInput();

  {
    int error = sftp.symlink("tolink", "linkfile");
    if (error) outputError(error, "symlink");

    std::string readlink;
    error = sftp.readlink(&readlink, "/tmp");
    if (error) outputError(error, "readlink");
  }

  waitUntilInput();

  {
    SFTP::Attrib attr;
    int error = sftp.stat(&attr, "./");
    if (error) {
      outputError(error, "stat1");
    } else {
      std::cout << " perm = " << attr.perm << "\n";
      if (! S_ISDIR(attr.perm)) {
        std::cout << " [ERROR] = " << "tmp" << " is not directory" << "\n";
      }
      std::cout << " atime = " << attr.atime << "\n";
    }

    error = sftp.stat(&attr, "/tmp");
    if (error) outputError(error, "stat2");

    error = sftp.lstat(&attr, "/tmp");
    if (error) outputError(error, "lstat1");

    error = sftp.lstat(&attr, "/tmp/Makefile");
    if (error) outputError(error, "lstat2");
  }

  waitUntilInput();

  {
    std::vector<SFTP::DirentPtr> dirent;
    int error = sftp.readdir(&dirent, "/tmp");
    if (error) {
      outputError(error, "readdir");
    } else {
      for (typeof(dirent.begin()) p = dirent.begin(); p != dirent.end(); ++p) {
        std::cout << " filename = " << (*p)->filename << "\n";
        std::cout << " longname = " << (*p)->longname << "\n";
      }
    }
  }

  waitUntilInput();

  {
    int error = sftp.mkdir("/tmp/mogemoge");
    if (error) outputError(error, "mkdir");

    error = sftp.rename("/tmp/mogemoge", "/tmp/mogefuga");
    if (error) outputError(error, "rename1");

    error = sftp.rename("/tmp/Makefile2", "/tmp/Makefile");
    if (error) outputError(error, "rename2");

    error = sftp.rename("/tmp/Makefile", "/tmp/mogefuga");
    if (error) outputError(error, "rename3");

    error = sftp.rename("/tmp/mogemoge", "/tmp/Makefile");
    if (error) outputError(error, "rename4");
  }

  sftp.terminate();

  return 0;
}
