#include "urdfs_kext.hpp"
extern "C" {
#include <sys/ubc.h>
#include <sys/fcntl.h>
#include <sys/dirent.h>
#include <sys/unistd.h>
}
#include "urdfs_kext_node.hpp"
#include "urdfs_kext_bridge.hpp"

int
URDFS::Kernel::getVnode(struct mount *mp, vnode_t dvp, int markroot,
                        struct componentname *cnp, ino_t ino, enum vtype vtype,
                        struct timespec atime, struct timespec mtime, struct timespec ctime,
                        off_t filesize,
                        vnode_t *vpp)
{
  START_MARKER(__PRETTY_FUNCTION__);

  vnode_t vp;

  // --------------------
  // from cache
  vp = URDFS::NodeHash::find(mp, ino);
  if (vp != NULLVP) {
    *vpp = vp;
    if (cnp->cn_flags & MAKEENTRY) {
      cache_enter(dvp, vp, cnp);
    }

    return RETURN(0, __PRETTY_FUNCTION__);
  }

  // --------------------
  // create new node
  URDFS::Node *new_pt;
  MALLOC(new_pt, URDFS::Node*, sizeof(URDFS::Node), M_TEMP, M_WAITOK);

  memset(new_pt, 0, sizeof(URDFS::Node));
  new_pt->ino = ino;
  new_pt->mountp = mp;
  new_pt->parent = dvp;
  new_pt->vnode = NULLVP;
  new_pt->cacheVnode = NULLVP;
  new_pt->atime = atime;
  new_pt->mtime = mtime;
  new_pt->ctime = ctime;
  new_pt->filesize = filesize;
  new_pt->opencount = 0;

  // --------------------
  // create vnode
  struct vnode_fsparam vfsp;
  vfsp.vnfs_mp = mp;
  vfsp.vnfs_vtype = vtype;
  vfsp.vnfs_str = URDFS::Bridge::fsname;
  vfsp.vnfs_dvp = dvp;
  vfsp.vnfs_fsnode = new_pt;
  vfsp.vnfs_vops = reinterpret_cast<int(**)(void*)>(urdfs_vnodeop_p);
  vfsp.vnfs_markroot = markroot;
  vfsp.vnfs_marksystem = 0;
  vfsp.vnfs_rdev = 0;
  vfsp.vnfs_filesize = filesize;
  vfsp.vnfs_cnp = cnp;
  vfsp.vnfs_flags = (dvp && cnp && (cnp->cn_flags & MAKEENTRY)) ? 0 : VNFS_NOCACHE;

  int error = vnode_create(VNCREATE_FLAVOR, VCREATESIZE, &vfsp, &vp);
  if (error) {
    FREE(new_pt, M_TEMP);

    return RETURN(error, __PRETTY_FUNCTION__);
  }

  // --------------------
  new_pt->vnode = vp;
  vnode_addfsref(vp);
  vnode_settag(vp, VT_OTHER);
  URDFS::NodeHash::add(new_pt);

  // --------------------
  *vpp = vp;

  return RETURN(0, __PRETTY_FUNCTION__);
}
