#ifndef URDFS_KEXT
#define URDFS_KEXT

extern "C" {
#include <sys/systm.h>
  errno_t sock_nointerrupt(socket_t so, int on);
#include <sys/vnode.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mount.h>
}

#include "urdfs_bridge.hpp"

namespace URDFS {
  namespace Kernel {
    extern const char *fsname;

    enum {
      UNKNOWNUID = 99,
    };

    struct MountArgs
    {
      user_addr_t mntfromname;
      int version;
      int socketNameLen;
      user_addr_t socketName;
      user_addr_t volName;
      u_int32_t flags;
      off_t dirSize;
    };

    struct MountInfo
    {
      vnode_t rootVnode;
      bool aliveFlag;
      struct mount *mountp;
      char *volName;
      struct sockaddr *socketName;
      off_t dirSize;
    };

    inline MountInfo *mount2mountinfo(struct mount *mp) {
      return reinterpret_cast<MountInfo *>(vfs_fsprivate(mp));
    }

    int getVnode(struct mount *mp, vnode_t dvp, int markroot,
                 struct componentname *cnp, ino_t ino, enum vtype vtype,
                 struct timespec atime, struct timespec mtime, struct timespec ctime,
                 off_t filesize, vnode_t *vpp);
  }
};

inline void
START_MARKER(const char *name) {
#ifdef DEBUG
  printf("[BEGIN] %s\n", name);
#else
#pragma unused(name)
#endif
}

inline int
RETURN(int error, const char *name) {
#ifdef DEBUG
  printf("[RETURN] [error = %d] %s\n", error, name);
#else
#pragma unused(error, name)
#endif
  return error;
}

extern int( **urdfs_vnodeop_p)();

#endif
