#include "urdfs_kext.hpp"
#include "urdfs_kext_node.hpp"

namespace URDFS {
  namespace NodeHash {
    static LIST_HEAD(hashhead, Node) *hashtbl = NULL;
    static u_long hashsize; // size of hash table - 1

    inline static hashhead *
    getHashTablePtr(struct mount *mountp, ino_t ino)
    {
      u_long idx = reinterpret_cast<u_long>(mountp) + ino;
      return hashtbl + (idx & hashsize);
    }

    void
    initialize(void)
    {
      hashtbl = reinterpret_cast<hashhead *>(hashinit(desiredvnodes, M_TEMP, &hashsize));
    }

    void
    terminate(void)
    {
      if (hashtbl != NULL) {
        FREE(hashtbl, M_TEMP);
      }
    }

    void
    add(Node *p)
    {
      LIST_INSERT_HEAD(getHashTablePtr(p->mountp, p->ino), p, hash);
    }

    void
    remove(Node *p)
    {
      LIST_REMOVE(p, hash);
    }

    vnode_t
    find(struct mount *mp, ino_t ino)
    {
      struct hashhead *hashptr = getHashTablePtr(mp, ino);
      for (Node *node = hashptr->lh_first; node != NULL; node = (node->hash).le_next) {
        if (vnode_mount(node->vnode) != mp || node->ino != ino) {
          continue;
        }

        vnode_t vp = node->vnode;
        uint32_t vid = vnode_vid(vp);
        if (vnode_getwithvid(vp, vid)) {
          return NULLVP;
        }
        return vp;
      }
      return NULLVP;
    }
  }
}
