#include "urdfs.hpp"
#include "urdfs_node.hpp"
#include "urdfs_mount.hpp"
#include "urdfs_server.hpp"
#include "urdfs_utils.hpp"
#include "sshfs_op.hpp"
#include "libsftp.hpp"

std::string kextpath = "/Applications/sshfs/urdfs.kext";

namespace {
  void
  canonMountPoint(std::string *mountpoint)
  {
    char real[PATH_MAX];
    realpath(mountpoint->c_str(), real);
    *mountpoint = real;
  }
}

int
main(int argc, char *argv[])
{
  if (argc != 3) {
    std::cerr << "Usage: " << argv[0] << " user@host:dir mountpoint" << "\n";
    exit(1);
  }

  umask(077);

  std::string sshhostpath = argv[1];
  std::string mountpoint = argv[2];
  canonMountPoint(&mountpoint);

  struct stat st;
  if (URDFS::Utils::stat(mountpoint, &st)) {
    std::cerr << "[ERROR] " << mountpoint << " is not directory" << "\n";
    exit(1);
  }

  SSHFS::Operator sshfsOp;
  if (! sshfsOp.initialize(sshhostpath, st.st_uid, st.st_gid)) {
    std::cerr << "[ERROR] can't connect SSH" << "\n";
    exit(1);
  }

  std::string mntfromname = sshhostpath;

  URDFS::Mount mnt;
  mnt.mount(&sshfsOp, mountpoint, mntfromname);

  return 0;
}
