#ifndef SSHFS_OP_HPP
#define SSHFS_OP_HPP

#include "urdfs.hpp"
#include "urdfs_node.hpp"
#include "urdfs_thread.hpp"
#include "urdfs/urdfs_op.hpp"

#include "libsftp.hpp"

namespace SSHFS {
  enum {
    IOSIZE = 4 * 1024,
    BLOCKSIZE = 4096,
    NUMBLOCKS = 1024LL * 1024LL * 1024LL * 1024LL / BLOCKSIZE, // 1024GB
    NUMFILES = 65535,

    STATCACHE_LIFETIME = 3,
  };

  class Operator : public URDFS::Operator {
  public:
    bool initialize(const std::string &sshhostpath, uid_t _uid, gid_t _gid);

    int create(const std::string &path);
    int open(const URDFS::NodeEntry &node, bool writeAccess, bool trunc);
    int getattr(struct stat *st, const std::string &path);
    int setattr(const struct stat &st, const std::string &path);
    int fsync(const std::string &path, const std::string &cacheFilePath);
    int mkdir(const std::string &path);
    int rmdir(const std::string &path);
    int rename(const std::string &newpath, const std::string &oldpath);
    int remove(const std::string &path);
    int statfs(struct vfsstatfs *statfs);
    int readdir(struct dirent **ent, int *entnum, URDFS::NodeEntry *node);
    int readlink(std::string *referred, const std::string &path);
    int symlink(const std::string &path, const std::string &target);

  private:
    void setuidgid(const std::string &path);
    void eraseStatCache(const std::string &path);

    SFTP::Handler sftp;
    uid_t uid;
    gid_t gid;

    struct StatCache {
      bool exist;
      struct stat st;
      time_t lifetime;
    };
    typedef boost::shared_ptr<StatCache> StatCachePtr;
    std::map<std::string, StatCachePtr> statcache;

    URDFS::Thread::Mutex mutex;
  };
}

#endif
