#ifndef URDFS_NODE_HPP
#define URDFS_NODE_HPP

#include <sys/types.h>
#include <string>
#include <map>
#include <boost/shared_ptr.hpp>

#include "urdfs_thread.hpp"
#include "urdfs_tmpfile.hpp"

namespace URDFS {
  class NodeEntry;
  typedef boost::shared_ptr<NodeEntry> NodeEntryPtr;

  class NodeEntry
  {
  public:
    // constructor
    static NodeEntry *makeRootNode(void);
    static NodeEntry *get(ino_t _ino);

    // ----------------------------------------
    // self
    ino_t getIno(void) const { return ino; }

    const std::string &getName(void) const { return name; }
    const std::string &getCacheFileName(void) const { return cache.getFileName(); }

    void getPath(std::string *path) const;
    void rename(NodeEntry *newParent, const std::string &newName);

    void deleteSelf(void);

    // ----------------------------------------
    // for children & parent
    NodeEntry *findChild(const std::string &_name);
    NodeEntry *makeChild(const std::string &_name);

    const NodeEntry *getParent(void) const { return parent; }

  private:
    NodeEntry(NodeEntry *_parent, const std::string &_name);
    void internal_deleteSelf(void);

    // ----------------------------------------
    ino_t ino;
    std::string name;

    URDFS::TmpFile cache;

    NodeEntry *parent;
    std::map<std::string, NodeEntry *> children;

    static ino_t nextIno;
    static URDFS::Thread::Mutex mutex;
    static std::map<ino_t, NodeEntryPtr> mapIno2Node;
  };
}

#endif
