#ifndef URDFS_SERVER_HPP
#define URDFS_SERVER_HPP

#include "urdfs.hpp"
#include "urdfs_op.hpp"
#include "urdfs_thread.hpp"

#include <sys/un.h>
#include <queue>
#include <string>

namespace URDFS {
  class Server {
  public:
    enum {
      NUM_THREAD = 5,
    };
    Server(URDFS::Operator *_op);
    ~Server(void);

    const struct sockaddr_un &getSockAddr(void) const { return listenSocketAddr; }
    void doLoop(void);

  private:
    bool makeSocket(void);
    void deleteSocket(void);

    void sendReply(int sock, void *data, size_t size, int error);
    void dispatchOperator(int sock);
    static void *handleRequestThread(void *arg);
    bool enqueueRequest(int sock);

    int do_lookup(URDFS::Bridge::Lookup::Reply *reply, const URDFS::Bridge::Lookup::Request &request);
    int do_create(URDFS::Bridge::Create::Reply *reply, const URDFS::Bridge::Create::Request &request);
    int do_open(URDFS::Bridge::Open::Reply *reply, const URDFS::Bridge::Open::Request &request);
    int do_close(const URDFS::Bridge::Close::Request &request);
    int do_getattr(URDFS::Bridge::GetAttr::Reply *reply, const URDFS::Bridge::GetAttr::Request &request);
    int do_setattr(const URDFS::Bridge::SetAttr::Request &request);
    int do_fsync(const URDFS::Bridge::Fsync::Request &request);
    int do_remove(const URDFS::Bridge::Remove::Request &request);
    int do_rename(const URDFS::Bridge::Rename::Request &request);
    int do_mkdir(URDFS::Bridge::MkDir::Reply *reply, const URDFS::Bridge::MkDir::Request &request);
    int do_rmdir(const URDFS::Bridge::RmDir::Request &request);
    int do_readdir(const URDFS::Bridge::ReadDir::Request &request);
    int do_readlink(URDFS::Bridge::ReadLink::Reply *reply, const URDFS::Bridge::ReadLink::Request &request);
    int do_symlink(URDFS::Bridge::SymLink::Reply *reply, const URDFS::Bridge::SymLink::Request &request);
    int do_statfs(URDFS::Bridge::StatFs::Reply *reply, const URDFS::Bridge::StatFs::Request &request);

    int syncCacheFileMode(const URDFS::NodeEntry &node);


    URDFS::Operator *op;
    std::queue<int> queueRequestSocket;
    URDFS::Thread::Mutex requestsMutex;
    URDFS::Thread::Cond requestCond;

    struct sockaddr_un listenSocketAddr;
    std::string listenSocketPath;
    int listenSocket;

    volatile bool exitLoopFlag;
  };
}

#endif
