#include "urdfs_tmpfile.hpp"
#include "urdfs_thread.hpp"
#include "urdfs.hpp"
#include <unistd.h>
#include <paths.h>
#include <sys/param.h>

namespace {
  std::string tmpBaseDir;
  URDFS::Thread::Mutex mutex;
  int opencount = 0;
}

void
URDFS::TmpFile::initializeTmpBaseDir(void)
{
  START_MARKER(__PRETTY_FUNCTION__);

  char pathbuf[] = _PATH_TMP "urdfs.XXXXXXXX";
  if (mkdtemp(pathbuf)) {
    tmpBaseDir = pathbuf;
  }
}

namespace {
  void
  terminateTmpBaseDir(void)
  {
    START_MARKER(__PRETTY_FUNCTION__);

    if (tmpBaseDir.size() > 0) {
      rmdir(tmpBaseDir.c_str());
    }
  }
}

const std::string &
URDFS::TmpFile::getTmpBaseDir(void)
{
  return tmpBaseDir;
}


// ----------------------------------------------------------------------
URDFS::TmpFile::TmpFile(void)
{
  mutex.lock();

  if (tmpBaseDir.size() == 0) {
    initializeTmpBaseDir();
  }

  char pathbuf[PATH_MAX];
  snprintf(pathbuf, sizeof(pathbuf), "%s/cache.XXXXXXXX", tmpBaseDir.c_str());

  int fd = mkstemp(pathbuf);
  if (fd == -1) {
    exit(1);
  }
  close(fd);

  path = pathbuf;

  ++opencount;

  mutex.unlock();
}

URDFS::TmpFile::~TmpFile(void)
{
  mutex.lock();

  if (path.size() > 0) {
    unlink(path.c_str());
  }

  --opencount;
  if (opencount == 0) {
    terminateTmpBaseDir();
  }

  mutex.unlock();
}
