#ifndef URDFS_UTILS_HPP
#define URDFS_UTILS_HPP

#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <unistd.h>

namespace URDFS {
  namespace Utils {
    inline enum vtype mode2vtype(mode_t m) {
      if (S_ISBLK(m)) return VBLK;
      if (S_ISCHR(m)) return VCHR;
      if (S_ISDIR(m)) return VDIR;
      if (S_ISFIFO(m)) return VFIFO;
      if (S_ISREG(m)) return VREG;
      if (S_ISLNK(m)) return VLNK;
      if (S_ISSOCK(m)) return VSOCK;
      return VNON;
    }

    inline u_int8_t mode2dtype(mode_t m) {
      if (S_ISFIFO(m)) return DT_FIFO;
      if (S_ISCHR(m)) return DT_CHR;
      if (S_ISDIR(m)) return DT_DIR;
      if (S_ISBLK(m)) return DT_BLK;
      if (S_ISREG(m)) return DT_REG;
      if (S_ISLNK(m)) return DT_LNK;
      if (S_ISSOCK(m)) return DT_SOCK;
      if (S_ISWHT(m)) return DT_WHT;
      return DT_UNKNOWN;
    }

    inline int stat(const std::string &path, struct stat *st) {
      return ::stat(path.c_str(), st);
    }

    inline int chown(const std::string &path, uid_t uid, gid_t gid) {
      return ::chown(path.c_str(), uid, gid);
    }

    inline int chmod(const std::string &path, mode_t mode) {
      return ::chmod(path.c_str(), mode);
    }
  }
}

#endif
